//Game.cpp -- this is the implementation of all the GameHandler classes
// member functions, as well as the implementation of all important functions
// of the program pertaining to game play.

#include "Game.h"
#include "Constants.h"
#include "GameView.h"
#include "GameWindow.h"
#include "NewSetWindow.h"
#include <SupportKit.h>

rgb_color PURPLE	= {255,  0,255,255};		//various color definitions
rgb_color BLUE      = {  0,  0,255,255};
rgb_color GREEN     = {  0,255,  0,255};
rgb_color YELLOW    = {255,127,  0,255};		//actually closer to orange
rgb_color RED		= {255,  0,  0,225};
rgb_color BLACK		= {  0,  0,  0,225};
uint8 WHITE     	= 63;

GameHandler::GameHandler()
{
  setupgame();
};

GameHandler::~GameHandler()
{
};

void GameHandler::KeyPress(char key)
{
  if (person[0].alive())
  switch (key)
  {
    case 'j':
      person[0].turn(true);
    break;
    case 'l':
      person[0].turn(false);
    break;
  };
};

void GameHandler::startnewgame()
{
  for (dummy = 0; dummy < 4; dummy++)
  {
    person[dummy].setup();
  };
  srand(clock());
  players = 4;
};

void GameHandler::pulse(BView *drawview, BWindow *ender)
{
  for (dummy = 0; dummy < 4; dummy++)
  {
    if (person[dummy].alive())
    {
      drawview->SetHighColor(person[dummy].drawcolor);
      drawview->StrokeEllipse(person[dummy].location,0,0);
      person[dummy].advance();
    };
  };
  if (players == 1)
    ender->PostMessage(new BMessage(MSG_ENDGAME));
};

void GameHandler::checkalive(GameBitmap *grid)
{
  for (dummy = 0; dummy < 4; dummy++)
  {
    if (person[dummy].alive())
    person[dummy].tokill = (grid->colorat(person[dummy].location) != WHITE);
  };
};

void GameHandler::explode(BView *drawview, BPoint location, int radius = 20, int forks = 10)
{
  drawview->SetHighColor(RED);
  BPoint end(0,0);
  for (int index = 0; index < forks; index++)
  {
    end.x = rand() % (2 * radius) + location.x - radius;
    end.y = rand() % (2 * radius) + location.y - radius;
    drawview->StrokeLine(location, end);
  };
};

void GameHandler::doexplosions(BView *drawview)
{
  for (dummy = 0; dummy < 4; dummy++)
  {
    if (person[dummy].tokill)
    {
      explode(drawview, person[dummy].location, 30);
      person[dummy].kill();
      players--;
    };
  }; 
};

void GameHandler::setupgame()
{
  person[0].name = "Roosevelt";
  person[0].human = true;
  person[0].drawcolor = BLUE;
  person[0].LTKey = 'j';
  person[0].RTKey = 'l';
  person[0].MNKey = 'k';
  person[0].JPKey = 'i';
  person[1].name = "Stalin";
  person[1].human = false;
  person[1].drawcolor = PURPLE;
  person[1].LTKey = 'a';
  person[1].RTKey = 'd';
  person[1].MNKey = 's';
  person[1].JPKey = 'w';
  person[2].name = "Hitler";
  person[2].human = false;
  person[2].drawcolor = YELLOW;
  person[2].LTKey = 'v';
  person[2].RTKey = 'n';
  person[2].MNKey = 'b';
  person[2].JPKey = 'g';
  person[3].name = "Churchill";
  person[3].human = false;
  person[3].drawcolor = GREEN;
  person[3].LTKey = '4';
  person[3].RTKey = '6';
  person[3].MNKey = '5';
  person[3].JPKey = '8';
};

//====================================================================
// functions from outside gamehandler

GameHandler Game;			//create the game.

void GameView::KeyDown(const char *bytes, int32 numBytes)
{
  Game.KeyPress(*bytes);
};

void GameWindow::MessageReceived(BMessage *message)
{
  switch (message->what)
  {
    case MSG_REFRESHSCREEN:
      Game.checkalive(fbmp_Game);
      if (fbmp_Game->Lock())
      {
        Game.doexplosions(fpic_Draw);
        Game.pulse(fpic_Draw, this);
        fpic_Draw->Sync();
        fbmp_Game->Unlock();
      };
      fpic_Main->DrawBitmap(fbmp_Game);
    break;
    case MSG_NEWSET:
      be_app->PostMessage(MSG_IMPORT);
    break;
    case MSG_NEWGAME:
      {  //sandwich brackets so the scope of "bounds" is confined.
      BRect bounds(20,20,20 + FIELD_WIDTH, 20 + FIELD_HEIGHT);
      if (fbmp_Game->Lock())
      { // this is for drawing all envionmental objects
        fpic_Draw->SetHighColor(BLACK);
        fpic_Draw->StrokeRect(bounds);	//this is for the outside barrier
        fbmp_Game->Unlock();
      };
      fmnu_Game->SetEnabled(false);
      fmnu_Help->SetEnabled(false);
      Game.startnewgame();
      SetPulseRate(REFRESH_RATE);
      };
    break;
    case MSG_ENDGAME:
      fmnu_Game->SetEnabled(true);
      fmnu_Help->SetEnabled(true);
      SetPulseRate(STOPPED_RATE);
    break;
    case MSG_QUIT:
      be_app->PostMessage(B_QUIT_REQUESTED);
    break;
    default:
      BWindow::MessageReceived(message);
  };
};

void NewSetWindow::Import()
{
  Show();
  if (Lock())
  {
    for (int index = 0; index < 4; index++)
    {
      ftxt_Names[index]->SetText(Game.person[index].name);
      fchk_Human[index]->SetValue(Game.person[index].human);
      fpic_KeyLT[index]->store = Game.person[index].LTKey;
      fpic_KeyRT[index]->store = Game.person[index].RTKey;
      fpic_KeyMN[index]->store = Game.person[index].MNKey;
      fpic_KeyJP[index]->store = Game.person[index].JPKey;
    };
    Unlock();
  };
};

void NewSetWindow::Export()
{
  for (int index = 0; index < 4; index++)
  {
    Game.person[index].name = (char *)ftxt_Names[index]->Text();
    Game.person[index].human = fchk_Human[index]->Value();
    Game.person[index].LTKey = fpic_KeyLT[index]->store;
    Game.person[index].RTKey = fpic_KeyRT[index]->store;
    Game.person[index].MNKey = fpic_KeyMN[index]->store;
    Game.person[index].JPKey = fpic_KeyJP[index]->store;
  };
  Hide();
};