//NewSetWindow.cpp -- basically sets up the New Set Window.

#include <SupportKit.h>
#include "NewSetWindow.h"

#define MSG_DONE 'done'
#define MSG_CLICK 'clck'

//clickkeyset member functions.

ClickKeySet::ClickKeySet(BRect bounds, const char *name)
  :BView(bounds, name, B_FOLLOW_NONE, B_WILL_DRAW)
{
  store = '~';
  highlighted = false;
};

void ClickKeySet::MouseDown(BPoint point)
{
  BMessage *clearmessage;
  clearmessage = new BMessage(MSG_CLICK);
  clearmessage->AddPointer("whose",this); 
  Window()->PostMessage(clearmessage);
  highlighted = true;
  MakeFocus();
  Invalidate();
};

void ClickKeySet::LoseFocus()
{
  highlighted = false;
  Invalidate();
}

void ClickKeySet::KeyDown(const char *bytes, int32 numbytes)
{
  if (highlighted)
  {
    store = bytes[0];
    highlighted = false;
    Invalidate();
  };
}; 

void ClickKeySet::Draw(BRect updaterect)
{
  rgb_color BLACK = {0,0,0,255};
  rgb_color WHITE = {255,255,255,255};
  BString display;
  if (highlighted)
  {
    SetHighColor(BLACK);
    FillRect(Bounds());
  }
  else
  {
    BString display("");
    BPoint stringplace(15,15);
    SetHighColor(WHITE);
    FillRect(Bounds());
    SetHighColor(BLACK);
    StrokeRect(Bounds());
    switch (store)
    {
      case B_BACKSPACE:
        display << "<-";
      break;
      case B_ENTER:
        display << "[ENT]";
      break;
      case B_SPACE:
        display << "[SPC]";
      break;
      case B_TAB:
        display << "[TAB]";
      break;
      case B_ESCAPE:
        display << "[ESC]";
      break;
      case B_LEFT_ARROW:
        display << "LEFT";
      break;
      case B_RIGHT_ARROW:
        display << "RIGHT";
      break;
      case B_UP_ARROW:
        display << "UP";
      break;
      case B_DOWN_ARROW:
        display << "DOWN";
      break;
      default:
        display << store;
    }
    stringplace.x -= StringWidth(display.String())/2;    
    DrawString(display.String(), stringplace);
  };
};

//newsetwindow member functions.

NewSetWindow::NewSetWindow()
  :BWindow(BRect(100,100,510,320),"Trench! Settings", 
    B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL,
    B_NOT_MOVABLE | B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_NOT_CLOSABLE)
{
  BRect bounds;
  int top, bottom;
  BString Name;
//create the master view
  
//drawing the first row (key)

  top = 20; bottom = 40;

//name title box
  bounds.Set(110,top,200,bottom);
  flbl_Name = new BStringView(bounds,"Name title","Name");
  AddChild(flbl_Name);
  
//human title box
  bounds.Set(205,top,245,bottom);
  flbl_Human = new BStringView(bounds,"Human title","Human");
  AddChild(flbl_Human);
  
//leftkey title box
  bounds.Set(250,top,280,bottom);
  flbl_Left = new BStringView(bounds,"Left title", "Left");
  AddChild(flbl_Left);
  
//rightkey title box
  bounds.Set(290,top,320,bottom);
  flbl_Right = new BStringView(bounds,"Right title", "Right");
  AddChild(flbl_Right);
  
//minekey title box
  bounds.Set(330,top,360,bottom);
  flbl_Mine = new BStringView(bounds, "Mine title", "Mine");
  AddChild(flbl_Mine);
  
//jumpkey title box
  bounds.Set(370,top,400,bottom);
  flbl_Jump = new BStringView(bounds, "Jump title", "Jump");
  AddChild(flbl_Jump);

//four players controls  
  for (int row = 0; row < 4; row++)
  {
//setting the row invariants.
    top = 50 + row * 30;
    bottom = 70 + row * 30;

//working with the names boxes.
    bounds.Set(10,top,200,bottom);
    Name = ""; Name << "Player " << row;
    ftxt_Names[row] = new BTextControl(bounds, Name.String(), Name.String(), "", NULL);
    AddChild(ftxt_Names[row]);

//Humanity check boxes.
    bounds.Set(210, top, 230, bottom);
    Name = ""; Name << "Human " << row;
    fchk_Human[row] = new BCheckBox(bounds,Name.String(), "", NULL);
    AddChild(fchk_Human[row]);

//Left key Click boxes
    bounds.Set(250, top, 280, bottom);
    Name = ""; Name << "Left " << row;
    fpic_KeyLT[row] = new ClickKeySet(bounds,Name.String());
    AddChild(fpic_KeyLT[row]);

//Right key Click boxes
    bounds.Set(290, top, 320, bottom);
    Name = ""; Name << "Right " << row;
    fpic_KeyRT[row] = new ClickKeySet(bounds,Name.String());
    AddChild(fpic_KeyRT[row]);
    
//Mine key Click boxes
    bounds.Set(330, top, 360, bottom);
    Name = ""; Name << "Mine " << row;
    fpic_KeyMN[row] = new ClickKeySet(bounds,Name.String());
    AddChild(fpic_KeyMN[row]);
    
//Jump key Click boxes
    bounds.Set(370, top, 400, bottom);
    Name = ""; Name << "Jump " << row;
    fpic_KeyJP[row] = new ClickKeySet(bounds,Name.String());
    AddChild(fpic_KeyJP[row]);    
  };
  
  bounds.Set(150,180,250,210);
  fcmd_Done = new BButton(bounds, "Done Button", "Done", new BMessage(MSG_DONE));
  AddChild(fcmd_Done);
};

void NewSetWindow::MessageReceived(BMessage *message)
{
  switch (message->what)
  {
    case MSG_DONE:
      Export();
    break;
    case MSG_CLICK:
      { //this is to sequester the declared variables.
        void **who; message->FindPointer("whose",who);
        for(int index = 0; index < 4; index++)
        {
          if (((ClickKeySet*) *who) != fpic_KeyLT[index])
            fpic_KeyLT[index]->LoseFocus();
          if (((ClickKeySet*) *who) != fpic_KeyRT[index])
            fpic_KeyRT[index]->LoseFocus();
          if (((ClickKeySet*) *who) != fpic_KeyMN[index])
            fpic_KeyMN[index]->LoseFocus();          
          if (((ClickKeySet*) *who) != fpic_KeyJP[index])
            fpic_KeyJP[index]->LoseFocus();
        }; 
      };
    break;
    default:
      BWindow::MessageReceived(message);
  };
};